@@ -1,12 +1,5 @@
-from typing import List
-import numpy as np
 import lmdb
-from jina import Document
-
-
-def dumps_doc(doc: Document):
-    new_doc = Document(doc, copy=True)
-    return new_doc.SerializeToString()
+import numpy as np
 
 
 def str2dtype(dtype_str: str):
